// drawtrades - Draw trades from a metatrader detailed report on a chart
// Be sure to put the statement file in experts/files
// Version 0.1

#property copyright "myke@omgcats.com"
#property show_inputs

extern string statement_filename = "DetailedStatement.htm";
extern int hour_shift = 0;

// Takes a filehandle, returns next line of file, or string "EOF"
string getline(int filehandle) {
  static string buffer = "";
  static int bytesread = 0;
  static int eolpos = -1;
  int blocksize = 80;
  string line;

  while((!FileIsEnding(filehandle)) && (eolpos == -1)) {
    line = FileReadString(filehandle, blocksize);
    buffer = StringConcatenate(buffer, line);
    eolpos = StringFind(buffer, "\n", 0);
  }
  if(eolpos == -1) {
    return("EOF");
  }
  line = StringTrimRight(StringSubstr(buffer, 0, eolpos));
  buffer = StringSubstr(buffer, eolpos + 1);
  eolpos = StringFind(buffer, "\n", 0);
  return(line);
}

// Returns the next table data element, or "" if not found, modifies
// input line in-place
string getnexttd(string& line) {
  int tagpos;
  string element;

  if(StringFind(line, "<td") == -1) {
    return("");
  }
  // Hopefully the next > is the end of td open tag!
  tagpos = StringFind(line, ">");
  if(tagpos == -1) {
    return("");
  }
  line = StringSubstr(line, tagpos + 1);
  // Hopefully the close tag is always </td>
  tagpos = StringFind(line, "</td>");
  if(tagpos == -1) {
    return("");
  }
  element = StringSubstr(line, 0, tagpos);
  line = StringSubstr(line, tagpos + 5);
  return(element);
}

// Takes a single line as input, tries to parse it as a detailed statement row
// If successful, returns meaningful values for its parameters and 0
// Otherwise returns -1
int parselineastablerow(string line, double& ticket, datetime& opentime, 
			datetime& closetime, string& transtype, string& pairname, 
			double& openprice, double& closeprice, double& stoploss, 
			double& takeprofit) {
  int tagpos;
  string element;

  // Clear out parameters since we don't know if we'll get valid values for them
  ticket = 0.0;
  opentime = 0.0;
  closetime = 0.0;
  openprice = 0.0;
  closeprice = 0.0;
  stoploss = 0.0;
  takeprofit = 0.0;
  transtype = "";
  pairname = "";

  // hardcoded table start pattern for results
  if(StringFind(line, "<tr align=right>", 0) == -1 &&
     StringFind(line, "<tr bgcolor=#E0E0E0 align=right>", 0) == -1) {
    return(-1);
  }
  // Lop off the tr tag, make gross assumptions about its format
  tagpos = StringFind(line, "align=right>", 0);
  line = StringSubstr(line, tagpos + 12);
  // Get the interesting elements and convert to return parameters
  element = getnexttd(line);
  if(element != "") {
    ticket = StrToInteger(element);
  }
  element = getnexttd(line);
  if(element != "") {
    opentime = StrToTime(element);
  }
  element = getnexttd(line);
  if(element != "") {
    transtype = element;
  }
  element = getnexttd(line);
  if(element != "") {
  }
  element = getnexttd(line);
  if(element != "") {
    pairname = element;
  }
  element = getnexttd(line);
  if(element != "") {
    openprice = StrToDouble(element);
  }
  element = getnexttd(line);
  if(element != "") {
    stoploss = StrToDouble(element);
  }
  element = getnexttd(line);
  if(element != "") {
    takeprofit = StrToDouble(element);
  }
  element = getnexttd(line);
  if(element != "") {
    closetime = StrToTime(element);
  }
  element = getnexttd(line);
  if(element != "") {
    closeprice = StrToDouble(element);
  }
  // Don't care about remaining elements in line
  return(0);
}

// Converts pair name if necessary
// This would be more efficient if pair info was cached
string convertpairname(string pairname) {
  string thispair = Symbol();
  int postfix = 0;
  int i;

  // Add postfix if this broker has 7 digit symbols, or
  // truncate report symbol if not
  if(StringLen(thispair) > 6) {
    postfix = StringGetChar(thispair, 7);
  } else {
    pairname = StringSubstr(pairname, 0, 6);
  }
  // Do upper/lower conversion
  if(StringGetChar(thispair, 0) >= 65 && 
     StringGetChar(thispair, 0) <= 90) {
    if(StringGetChar(pairname, 0) >= 97 &&
       StringGetChar(pairname, 0) <= 122) {
      for(i = 0; i < StringLen(pairname); i++) {
	pairname = StringSetChar(pairname, i, StringGetChar(pairname, i) - 32);
      }
    }
  } else if(StringGetChar(pairname, 0) >= 65 &&
	    StringGetChar(pairname, 0) <= 90) {
    for(i = 0; i < StringLen(pairname); i++) {
      pairname = StringSetChar(pairname, i, StringGetChar(pairname, i) + 32);
    }
  }
  if(postfix > 0) {
    pairname = StringConcatenate(pairname, CharToStr(postfix));
  }
  return(pairname);
}

// Draw trade on chart, if possible
void drawtrade(int ticket, string transtype, datetime opentime,
	       double openprice, datetime closetime, double closeprice,
	       double stoploss, double takeprofit) {
  color linecolor;
  int linestyle;
  color maxcolor;
  color mincolor;
  double maxlevel;
  double minlevel;
  string leftarrowname = StringConcatenate("leftarrow-", ticket);
  string linename = StringConcatenate("line-", ticket);
  string rightarrowname = StringConcatenate("rightarrow-", ticket);
  string maxarrowname = StringConcatenate("maxarrow-", ticket);
  string minarrowname = StringConcatenate("minarrow-", ticket);

  opentime += 3600 * hour_shift;
  closetime += 3600 * hour_shift;

  if(transtype == "buy" || transtype == "buy limit" || transtype == "buy stop") {
    linecolor = Blue;
    maxcolor = Blue;
    maxlevel = takeprofit;
    mincolor = Red;
    minlevel = stoploss;
  } else if(transtype == "sell" || transtype == "sell limit" || transtype == "sell stop") {
    linecolor = Red;
    maxcolor = Red;
    maxlevel = stoploss;
    mincolor = Blue;
    minlevel = takeprofit;
  }
  if(transtype == "buy limit" || transtype == "sell limit" || 
     transtype == "buy stop" || transtype == "buy limit") {
    linestyle = STYLE_DASH;
  } else {
    linestyle = STYLE_DOT;
  }

  ObjectCreate(leftarrowname, OBJ_ARROW, 0, opentime, openprice);
  ObjectSet(leftarrowname, OBJPROP_ARROWCODE, 2);
  ObjectCreate(linename, OBJ_TREND, 0, opentime, openprice, closetime, closeprice);
  ObjectSet(linename, OBJPROP_STYLE, linestyle);
  ObjectSet(linename, OBJPROP_RAY, false);
  ObjectSet(linename, OBJPROP_COLOR, linecolor);
  ObjectCreate(rightarrowname, OBJ_ARROW, 0, closetime, closeprice);
  ObjectSet(rightarrowname, OBJPROP_ARROWCODE, 3);
  ObjectCreate(maxarrowname, OBJ_ARROW, 0, opentime, maxlevel);
  ObjectSet(maxarrowname, OBJPROP_ARROWCODE, 4);
  ObjectSet(maxarrowname, OBJPROP_COLOR, maxcolor);
  ObjectCreate(minarrowname, OBJ_ARROW, 0, opentime, minlevel);
  ObjectSet(minarrowname, OBJPROP_ARROWCODE, 4);
  ObjectSet(minarrowname, OBJPROP_COLOR, mincolor);
}


int start() {
  int fh;
  string line;
  int count = 0;
  // wish there was a more elegant way to do this
  datetime opentime, closetime;
  string transtype, pairname;
  double ticket, openprice, stoploss, takeprofit, closeprice;

  fh = FileOpen(statement_filename, FILE_READ|FILE_BIN);
  if(fh < 1) {
    Print("Couldn't open ", statement_filename, ": ", GetLastError());
    return(-1);
  }
  line = getline(fh);
  while(line != "EOF") {
    count++;
    if(parselineastablerow(line, ticket, opentime, closetime, transtype,
			   pairname, openprice, closeprice,
			   stoploss, takeprofit) == 0) {
      pairname = convertpairname(pairname);
      if(pairname == Symbol()) {
	Print("Drawing trade ", ticket);
	drawtrade(ticket, transtype, opentime, openprice, closetime,
		  closeprice, stoploss, takeprofit);
      }
    }
    line = getline(fh);
  }
  FileClose(fh);
}
